#!/usr/local/perl/bin/perl -w

# Favicon - Adds favicons to comments and trackbacks
# Henrik Gemal - http://gemal.dk/mt/favicon.html
# If you modify this code please let me know and I'll try to incorporate the changes into the next version

package Favicon;

use strict;
use warnings;
use MT::App;

use vars qw($VERSION $DEBUG @ISA);
$VERSION = "1.4";
$DEBUG = 0;
@ISA = qw(MT::App);

sub init {
	my $app = shift;
	$app->SUPER::init(@_) or return;
	$app->add_methods (
			default	=> \&default,
			remove	=> \&remove,
			remove_all	=> \&remove_all,
	);
	$app->{default_mode} = "default";
	$app->{requires_login} = 1;
	$app->{user_class} = "MT::Author";
	$app->{is_admin} = 1;
	$app;
}

sub favicon_debug {
	my($txt) = @_;
	print STDERR "MTFavicon: $txt\n" if ($DEBUG);
}

sub default {
	my ($app, $html, $parent, $child) = @_;
	use Data::Dumper;
	my %param = (func_default => 1, version => $VERSION);

	$param{count} = MT::PluginData->count({plugin => "Favicon"});

	my %entries;
	my %entries_not;
	foreach my $data (MT::PluginData->load({plugin => "Favicon"})) {
		if (exists $entries{$data->key} || exists $entries_not{$data->key}) {
			favicon_debug("Arghh!" . $data->key . ":" . $data->id);
		}
		if (!exists ${$data->data}->{favicon}) {
			$entries_not{$data->id} = $data->data;
		} else {
			$entries{$data->id} = $data->data;
		}
	}

	my @favicons;
	foreach my $data (sort keys %entries) {
		my $data_url = ${$entries{$data}}->{url} || undef;
		my $data_url_short = substr($data_url, 0, 50) || undef;
		my $data_favicon = ${$entries{$data}}->{favicon} || undef;
		my $data_date = ${$entries{$data}}->{date} || undef;
		push @favicons, {id => $data, url => $data_url, url_short => $data_url_short, favicon => $data_favicon, date => $data_date};
	}
	$param{favicons} = \@favicons;

	my @favicons_not;
	foreach my $data (sort keys %entries_not) {
		my $data_url = ${$entries_not{$data}}->{url} || undef;
		my $data_url_short = substr($data_url, 0, 60) || undef;
		my $data_favicon = ${$entries_not{$data}}->{favicon} || undef;
		my $data_date = ${$entries_not{$data}}->{date} || undef;
		my $data_date_diff = int(time - $data_date);
		push @favicons_not, {id => $data, url => $data_url, url_short => $data_url_short, favicon => $data_favicon, date => $data_date, date_diff => $data_date_diff};
	}
	$param{favicons_not} = \@favicons_not;

	$app->add_breadcrumb($app->translate("Favicon"));
	$app->build_page("favicon.tmpl", \%param);
}

sub remove {
	my ($app, $html, $parent, $child) = @_;
	my $id = $app->{query}->param("id") || "";

	if ($id) {
		my $data = MT::PluginData->load({plugin => "Favicon", id => $id});
		if ($data) {
			favicon_debug("<$id> <" . $data->key . "> removed");
			$data->remove or die "Removing favicon failed: ", $data->errstr;
		} else {
			favicon_debug("<$id> not found");
		}
	}
	$app->redirect($app->base . "/plugins/Favicon/favicon.cgi");
}

sub remove_all {
	my ($app, $html, $parent, $child) = @_;

	foreach my $data (MT::PluginData->load({plugin => "Favicon"})) {
		$data->remove or die "Removing favicon failed: ", $data->errstr;
	}
	$app->redirect($app->base . "/plugins/Favicon/favicon.cgi");
}
